package com.atlassian.confluence.rpc;

import com.atlassian.user.UserManager;
import com.atlassian.user.GroupManager;
import com.atlassian.user.User;
import com.atlassian.user.Group;
import com.atlassian.user.security.password.PasswordEncryptor;
import com.atlassian.user.security.authentication.Authenticator;
import com.atlassian.user.security.authentication.DefaultAuthenticator;
import com.atlassian.user.search.page.Pager;
import com.atlassian.user.search.query.EntityQueryParser;
import com.atlassian.user.search.query.DefaultEntityQueryParser;
import com.atlassian.user.configuration.Configuration;
import com.atlassian.user.configuration.DelegationAccessor;
import com.atlassian.user.configuration.RepositoryAccessor;
import com.atlassian.user.configuration.DefaultDelegationAccessor;
import com.atlassian.user.properties.PropertySetFactory;
import com.atlassian.user.repository.Repository;
import com.atlassian.user.repository.DefaultRepository;
import com.atlassian.user.impl.memory.provider.MemoryProvider;
import com.atlassian.user.impl.memory.MemoryUserManager;
import com.atlassian.user.impl.memory.MemoryGroupManager;
import com.atlassian.user.impl.memory.properties.MemoryPropertySetFactory;
import com.atlassian.user.impl.osuser.security.password.OSUPasswordEncryptor;
import com.atlassian.confluence.user.UserAccessor;
import com.atlassian.confluence.user.DefaultUserAccessor;
import com.atlassian.confluence.user.PersonalInformationManager;
import com.atlassian.confluence.user.PersonalInformation;
import com.atlassian.confluence.user.persistence.dao.PersonalInformationDao;
import com.atlassian.confluence.security.SpacePermissionManager;
import com.atlassian.confluence.security.SpacePermission;
import com.atlassian.confluence.spaces.Space;
import com.atlassian.confluence.core.*;
import com.atlassian.confluence.mail.notification.NotificationManager;
import com.atlassian.confluence.mail.notification.Notification;
import com.atlassian.confluence.pages.AbstractPage;
import com.atlassian.event.*;

import java.util.*;

/**
 * UserTestHelper
 * nickf
 * Dec 13, 2004
 * 11:17:56 AM
 */

public class UserTestHelper
{
    static UserManager userManager;
    public static GroupManager groupManager;
    public static UserAccessor userAccessor;
    public static MemoryProvider provider;
    public static Repository repository;
    public static PropertySetFactory propertySetFactory;

    static Configuration config;
    private static DelegationAccessor delegationAccessor;

    public static PropertySetFactory getPropertySetFactory()
    {
        return UserTestHelper.delegationAccessor.getPropertySetFactory();
    }

    public static UserManager getUserManager()
    {
        return UserTestHelper.delegationAccessor.getUserManager();
    }

    public static GroupManager getGroupManager()
    {
        return UserTestHelper.delegationAccessor.getGroupManager();
    }

    public static UserAccessor getUserAccessor()
    {
        return UserTestHelper.userAccessor;
    }

    public static UserAccessor getTestUserAccessor()
    {
        if (UserTestHelper.userAccessor == null)
        {
            DefaultUserAccessor userAccessor = new DefaultUserAccessor();
            userAccessor.setAtlassianUserConfiguration(UserTestHelper.getTestConfiguration());
            userAccessor.setSpacePermissionManager(UserTestHelper.getAnonSpacePermissionManager());
            userAccessor.setEventManager(UserTestHelper.getAnonEventManager());
            userAccessor.setNotificationManager(UserTestHelper.getAnonNotificationManager());
            userAccessor.setPersonalInformationManager(UserTestHelper.getAnonPersonalInformationManager(userAccessor));
            userAccessor.setContentEntityManager(UserTestHelper.getAnonContentEntityManager());
            userAccessor.setUserChecker(new BogusUserChecker());

            UserTestHelper.userAccessor = userAccessor;
        }

        return UserTestHelper.userAccessor;
    }

    public static Configuration getTestConfiguration()
    {
        if (UserTestHelper.config == null)
        {
            if (UserTestHelper.delegationAccessor == null)
            {
                UserTestHelper.delegationAccessor = UserTestHelper.getTestDelegationAccessor();
            }

            UserTestHelper.config = UserTestHelper.getAnonAtlassianUserConfiguration(UserTestHelper.delegationAccessor);
        }

        return UserTestHelper.config;
    }

    private static Configuration getAnonAtlassianUserConfiguration(final DelegationAccessor delegationAccessor)
    {
        return new Configuration()
        {

            public RepositoryAccessor getRepositoryAccessor()
            {
                return delegationAccessor;
            }

            public DelegationAccessor getDelegationAccessor()
            {
                return delegationAccessor;
            }

            public void init()
            {
            }

            public boolean isInitialized()
            {
                return true;
            }
        };
    }


    public static SpacePermissionManager getAnonSpacePermissionManager()
    {
        return new SpacePermissionManager(){
            public void savePermission(SpacePermission permission)
            {
            }

            public void removeAllPermissions(Space space)
            {

            }

            public void removePermission(SpacePermission permission)
            {
            }

            public boolean hasPermission(String permissionType, Space space, User remoteUser)
            {
                return false;
            }

            public boolean hasPermission(List permissionTypes, Space space, User remoteUser)
            {
                return false;
            }

            public List getGlobalPermissions()
            {
                return new ArrayList();
            }

            public void removeAllUserPermissions(String username)
            {
            }

            public void removeAllPermissionsForGroup(String group)
            {
            }

            public List getGlobalPermissions(String permissionType)
            {
                  return new ArrayList();
            }

            public void fixPermissions()
            {
            }

            public void flushCaches()
            {
            }

            public boolean groupHasPermission(String permissionType, Space space, String group)
            {
                return false;
            }

            public void createInitialSpacePermissions(Space space, User creator)
            {
            }

            public boolean hasPermissionForSpace(User user, List permissionTypes, Space space)
            {
                return false;
            }

            public Collection getGroupsWithPermissions(Space space)
            {
                return null;
            }

            public Map getGroupsForPermissionType(String permissionType, Space space)
            {
                return null;
            }

            public Collection getUsersWithPermissions(Space space)
            {
                return null;
            }

            public Map getUsersForPermissionType(String permissionType, Space space)
            {
                return null;
            }
        };
    }

    public static void removeAllTestEntities(UserAccessor userAccessor)
    {
        userAccessor.setContentEntityManager(UserTestHelper.getAnonContentEntityManager());
        userAccessor.setEventManager(UserTestHelper.getAnonEventManager());
        userAccessor.setNotificationManager(UserTestHelper.getAnonNotificationManager());
        userAccessor.setPersonalInformationManager(UserTestHelper.getAnonPersonalInformationManager(userAccessor));
        userAccessor.setSpacePermissionManager(UserTestHelper.getAnonSpacePermissionManager());

        Pager userPager = userAccessor.getUsers();

        Iterator userIterator = userPager.iterator();

        while (userIterator.hasNext())
        {
            userAccessor.removeUser((User) userIterator.next());
        }

        Pager groupPager = userAccessor.getGroups();

        Iterator groupIterator = groupPager.iterator();

        while (groupIterator.hasNext())
        {
            Group group = (Group) groupIterator.next();
            userAccessor.removeGroup(group);
        }
    }

    public static EventManager getAnonEventManager()
    {
        return new NoOpEventManager();
    }

    public static PersonalInformationManager getAnonPersonalInformationManager(final UserAccessor userAccessor)
    {
        return new PersonalInformationManager(){
            public PersonalInformation getPersonalInformation(String username)
            {
                return new PersonalInformation();
            }

            public PersonalInformation getPersonalInformation(User user)
            {
                return new PersonalInformation();
            }

            public void savePersonalInformation(PersonalInformation newInfo, PersonalInformation oldInfo)
            {
            }

            public void setContentEntityManager(ContentEntityManager contentEntityManager)
            {
            }

            public void setPersonalInformationDao(PersonalInformationDao personalInformationDao)
            {
            }

            public void removePersonalInformation(String username)
            {
            }

            public PersonalInformation createPersonalInformation(User user)
            {
                return new PersonalInformation(user, userAccessor);
            }
        };
    }

    public static NotificationManager getAnonNotificationManager()
    {
        return new NotificationManager(){
            public List getNotificationsByUser(User user)
            {
                return null;
            }

            public List getNotificationsByPage(AbstractPage page)
            {
                return null;
            }

            public List getNotificationsBySpace(Space space)
            {
                return null;
            }

            public Notification getNotificationByUserAndPage(User user, AbstractPage page)
            {
                return null;
            }

            public Notification getNotificationByUserAndSpace(User user, String spaceKey)
            {
                return null;
            }

            public Notification addPageNotification(User user, AbstractPage page)
            {
                return null;
            }

            public Notification addSpaceNotification(User user, Space space)
            {
                return null;
            }

            public Notification addDailyReportNotfication(User user)
            {
                return null;
            }

            public void removeDailyReportNotification(User user)
            {
            }

            public void removeNotification(Notification notification)
            {
            }

            public boolean isUserWatchingPageOrSpace(User user, Space space, AbstractPage page)
            {
                return false;
            }

            /**
             * Remove all of the notifications setup for this user. This includes space, page and daily
             * report notifications.
             *
             * @param user
             */
            public void removeAllNotificationsForUser(User user)
            {
            }

            public List getDailyReportNotifications()
            {
                return null;
            }

            public Notification getDailyReportNotificationForUser(User user)
            {
                return null;
            }
        };
    }

    public static ContentEntityManager getAnonContentEntityManager()
    {
        return new ContentEntityManager(){
            /**
             * @param obj         - the  <code>ContentEntityObject</code> to save
             * @param saveContext - a <code>SaveContext</code> object for the manager.
             */
            public void saveContentEntity(ContentEntityObject obj, SaveContext saveContext)
            {
            }

            /**
             * In order to create a history for this object, we need to pass in the modified as well as the original version of the object.
             *
             * @param obj
             * @param origObj
             * @param saveContext - a <code>SaveContext</code> holding additional parameters for the manager to use when saving.
             */
            public void saveContentEntity(ContentEntityObject obj, ContentEntityObject origObj, SaveContext saveContext)
            {
            }

            public void removeContentEntity(ContentEntityObject obj)
            {
            }

            public void refreshContentEntity(ContentEntityObject obj)
            {
            }

            public void revertContentEntityBackToVersion(ContentEntityObject obj, int version)
            {
            }

            /**
             * Retrieve an iterator of recently added entities, in order of most recent to last.
             * The iterator is lazy-loading, so you can filter the output and return the first 'n'
             * that interest you.
             * <p/>
             * For performance reasons, it's a VERY bad idea to supply both a space key, and infinite
             * results.
             * <p/>
             * If you supply a space key, this WILL NOT RETURN ANY COMMENTS. There is currently no
             * way around this, you have to retrieve comments separately and splice the results together.
             * Sorry.
             *
             * @param spaceKey   the space key to look up the entities for. If spaceKey is null, there
             *                   is assumed to be no space, and things like user info and space descriptions will
             *                   also be returned.
             * @param maxResults the maximum number of entities to return in the iterator. Any number
             *                   zero or less (Use the ITERATE_ALL constant) will cause the iterator to go over
             *                   every entity.
             */
            public Iterator getRecentlyAddedEntities(String spaceKey, int maxResults)
            {
                return null;
            }

            /**
             * Retrieve an iterator of recently modified entities, in order of most recent to last.
             * The iterator is lazy-loading, so you can filter the output and return the first 'n'
             * that interest you.
             * <p/>
             * For performance reasons, it's a VERY bad idea to supply both a space key, and infinite
             * results.
             *
             * @param spaceKey   the space key to look up the entities for.
             * @param maxResults the maximum number of entities to return in the iterator. Any number
             *                   zero or less (Use the ITERATE_ALL constant) will cause the iterator to go over
             *                   every entity.
             */
            public Iterator getRecentlyModifiedEntities(String spaceKey, int maxResults)
            {
                return null;
            }

            /**
             * @deprecated for content queries, use the list manager instead.
             */
            public Iterator getRecentlyModifiedEntities(int maxResults)
            {
                return null;
            }

            public Iterator getAllCurrentEntities()
            {
                return null;  //To change body of implemented methods use File | Settings | File Templates.
            }

            /**
             *
             */
            public List getEntitiesModifiedSince(Date date)
            {
                return null;
            }

            /**
             * Retrieve an iterator of recently modified entities, in order of most recent to last.
             * The iterator is lazy-loading, so you can filter the output and return the first 'n'
             * that interest you.
             *
             * @param username the user for which to retrieve the modified content
             */
            public Iterator getRecentlyModifiedEntitiesForUser(String username)
            {
                return null;
            }

            public Iterator getRecentlyModifiedEntitiesByType(ListQuery query, int firstResult)
            {
                return null;
            }

            public List getLockedBySpace(String spaceKey)
            {
                return null;
            }

            public ContentEntityObject getById(long id)
            {
                return null;
            }

            /**
             * Loops through all the content that currently links to 'contentBeingChanged' and renames all the links. <br>
             * If you only want to change the links of a specific content object, use the 'updateSingleContentLinkinTo' function. <br>
             * This rename will be recorded in the content's history
             *
             * @param contentBeingRefactored
             * @param newSpaceKey
             * @param newTitle
             */
            public void updateContentLinkingTo(SpaceContentEntityObject contentBeingRefactored, String newSpaceKey, String newTitle)
            {
            }

            /**
             * Update the links on the page itself. If links are local (refering to the current space), a spacekey has to be added to the link
             *
             * @param contentBeingRefactored
             * @param newSpaceKey
             */
            public void updateOutgoingLinksInContent(SpaceContentEntityObject contentBeingRefactored, String newSpaceKey)
            {
            }

            /**
             * Renames all links contained in the 'referringContent' that currently links to 'contentBeingRefactored'. <br>
             * This rename will be recorded in the content's history
             *
             * @param referringContent
             * @param contentBeingRefactored
             * @param newSpaceKey
             * @param newTitle
             */
            public void updateSingleContentLinkinTo(ContentEntityObject referringContent, SpaceContentEntityObject contentBeingRefactored, String newSpaceKey, String newTitle)
            {
            }

            public void updateSingleContentLinkinTo(ContentEntityObject referringContent, SpaceContentEntityObject contentBeingRefactored, String newSpaceKey, String newTitle, SaveContext saveContext) {

            }

            public void setEventManager(EventManager eventManager)
            {
            }

            public ContentEntityObject getPreviousVersion(ContentEntityObject ceo)
            {
                return null;
            }

            public ContentEntityObject getNextVersion(ContentEntityObject ceo)
            {
                return null;
            }

            public ContentEntityObject getOtherVersion(ContentEntityObject ceo, int version)
            {
                return null;
            }

            public List getVersionHistorySummaries(ContentEntityObject ceo)
            {
                return null;
            }
        };
    }

    public static void createDefaultGroups()
    {
        if (UserTestHelper.userAccessor == null)
            UserTestHelper.userAccessor = UserTestHelper.getTestUserAccessor();

        try
        {
            UserTestHelper.userAccessor.createGroup(UserAccessor.GROUP_CONFLUENCE_ADMINS);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        try
        {
            UserTestHelper.userAccessor.createGroup(UserAccessor.GROUP_CONFLUENCE_USERS);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    private static DelegationAccessor getTestDelegationAccessor()
    {
        final DefaultRepository repository = new DefaultRepository();
        MemoryProvider provider = new MemoryProvider();
        final PasswordEncryptor encryptor = new OSUPasswordEncryptor();
        final MemoryUserManager userManager = new MemoryUserManager(repository, provider, encryptor);
        final MemoryGroupManager groupManager = new MemoryGroupManager(repository, provider);
        final PropertySetFactory propertySetFactory = new MemoryPropertySetFactory(repository);
        final Authenticator authenticator = new DefaultAuthenticator(userManager, encryptor);
        final EntityQueryParser parser = new DefaultEntityQueryParser(repository, userManager, groupManager);

        RepositoryAccessor accessor = new RepositoryAccessor()
        {
            public Repository getRepository()
            {
                return repository;
            }

            public UserManager getUserManager()
            {
                return userManager;
            }

            public GroupManager getGroupManager()
            {
                return groupManager;
            }

            public PropertySetFactory getPropertySetFactory()
            {
                return propertySetFactory;
            }

            public Authenticator getAuthenticator()
            {
                return authenticator;
            }

            public EntityQueryParser getEntityQueryParser()
            {
                return parser;
            }
        };

        DelegationAccessor delegation = new DefaultDelegationAccessor();
        delegation.addRepositoryAccessor("default", accessor);
        delegation.orderDelegation(new String[]{"default"});

        return delegation;
    }

}
